#include <IOKit/IOLib.h>
#include <IOKit/IORegistryEntry.h>
#include <IOKit/IOKitKeys.h>
#include <IOKit/IOService.h>
#include <IOKit/pci/IOPCIDevice.h>
#include "Natit.h"

OSDefineMetaClassAndStructors(com_dmweb_Natit, IOService)

#define super IOService


bool
com_dmweb_Natit::init( OSDictionary *properties )
{
    IOService::init(properties);
	IOLog("Natit: Starting.\n");
	IORegistryEntry *aNode = 0;
	IORegistryIterator *iter;
	OSObject *tObj1;
	OSObject *tObj2;
	OSString *tStr;
	OSData *tDat;
	UInt32 NVVenId = 0x10de;
	bool matchFound = false;
	
	iter = IORegistryIterator::iterateOver(IORegistryEntry::getPlane(kIODeviceTreePlane), kIORegistryIterateRecursively);
	if (iter) {
		IOLog("Natit: Iterating IORegistry.\n");
		aNode = iter->getNextObject();
		while (aNode) {
			tObj1 = aNode->getProperty("name");
			tObj2 = aNode->getProperty("vendor-id");
			
			if (tObj1 && tObj2) {
				tStr = OSDynamicCast(OSString, tObj1);
				tDat = OSDynamicCast(OSData, tObj2);
			    if (tStr && tDat) {
					if (tStr->isEqualTo("display") && tDat->isEqualTo(&NVVenId,2)) {
						matchFound = true;
						IOLog("Natit: Match found.\n");
						injectValues(aNode);
						break;
					}
				}
			}
			aNode = iter->getNextObject();
		}
		iter->release();
		if (!matchFound) {
			IOLog("Natit: Didn't find a match.\n");
		}
	} else {
		IOLog("Natit: Couldn't get iterator.\n");
	}
	return true;
}

bool
com_dmweb_Natit::start( IOService * provider )
{
	return true;
}

IOService* 
com_dmweb_Natit::probe(IOService *provider, SInt32 *score)
{
	return 0;
}

bool 
com_dmweb_Natit::attach(IOService *provider)
{
    return super::attach(provider);
}

void 
com_dmweb_Natit::detach(IOService *provider)
{
    // Like attach, this method may be called multiple times
    return super::detach(provider);
}

void
com_dmweb_Natit::injectValues(IORegistryEntry *theNode) {
	char *values[][3] = {
		{ "String", "device_type", "NVDA,Parent" },
		{ "String", "@0,device_type", "display" },
		{ "String", "@1,device_type", "display" },
//		{ "String", "@0,display-type", "LCD" },
//		{ "String", "@1,display-type", "CRT" },
		{ "String", "@0,compatible", "NVDA,NVMac"},
		{ "String", "@1,compatible", "NVDA,NVMac"},
		{ "String", "@0,name", "NVDA,Display-A"},
		{ "String", "@1,name", "NVDA,Display-B"},
		{ "UInt32", "@0,fboffset", "0x20000" }
	};
		
	OSString *modelName;
	OSData *deviceId;
	OSString *tmpOSString;
	char tmpString[6];
	int nrProps = sizeof(values)/sizeof(values[0]);
	int i;
	UInt32 tmpUI32;
	OSObject *tmpObj;
	
	for (i=0; i < nrProps; i++) {
		IOLog("Natit: Setting %s=%s\n", values[i][1], values[i][2]);
		if (!strcmp(values[i][0],"String")) {
			tmpObj = OSData::withBytes(values[i][2], strlen(values[i][2])+1);
		} else if (!strcmp(values[i][0],"UInt32")) {
			sscanf(values[i][2], "%i", &tmpUI32); 
			tmpObj = OSData::withBytes(&tmpUI32, sizeof(UInt32));
		}
		
		theNode->setProperty(values[i][1], tmpObj);
	}
	
	tmpObj = theNode->getProperty("device-id");
	if (tmpObj) {
		deviceId = OSDynamicCast(OSData, tmpObj);
		sprintf(tmpString, "%#06X", *(UInt32 *)deviceId->getBytesNoCopy());
		tmpOSString = OSString::withCString(tmpString);
		modelName = getModelName(tmpOSString);
		IOLog("Natit: Setting model=%s\n", modelName->getCStringNoCopy());
		tmpObj = OSData::withBytes(modelName->getCStringNoCopy(), modelName->getLength()+1);
		theNode->setProperty("model",tmpObj);
	}
	return;
}

OSString *
com_dmweb_Natit::getModelName(OSString *venId) {
	char *modelNames[][2] = {
		{ "0x014F",  "GeForce 6200"}  ,
		{ "0x00F3",  "GeForce 6200"}  ,
		{ "0x0221",  "GeForce 6200"}  ,
		{ "0x0163",  "GeForce 6200 LE"}  ,
		{ "0x0162",  "GeForce 6200SE TurboCache(TM)"}  ,
		{ "0x0161",  "GeForce 6200 TurboCache(TM)"}  ,
		{ "0x0162",  "GeForce 6200SE TurboCache(TM)"}  ,
		{ "0x0160",  "GeForce 6500"}  ,
		{ "0x0141",  "GeForce 6600"}  ,
		{ "0x00F2",  "GeForce 6600"}  ,
		{ "0x0140",  "GeForce 6600 GT"}  ,
		{ "0x00F1",  "GeForce 6600 GT"}  ,
		{ "0x0142",  "GeForce 6600 LE"}  ,
		{ "0x00F4",  "GeForce 6600 LE"}  ,
		{ "0x0143",  "GeForce 6600 VE"}  ,
		{ "0x0147",  "GeForce 6700 XL"}  ,
		{ "0x0041",  "GeForce 6800"}  ,
		{ "0x00C1",  "GeForce 6800"}  ,
		{ "0x0047",  "GeForce 6800 GS"}  ,
		{ "0x00F6",  "GeForce 6800 GS"}  ,
		{ "0x00C0",  "GeForce 6800 GS"}  ,
		{ "0x0045",  "GeForce 6800 GT"}  ,
		{ "0x00F9",  "GeForce 6800 Series GPU"}  ,
		{ "0x00C2",  "GeForce 6800 LE"}  ,
		{ "0x0040",  "GeForce 6800 Ultra"}  ,
		{ "0x00F9",  "GeForce 6800 Series GPU"}  ,
		{ "0x0043",  "GeForce 6800 XE"}  ,
		{ "0x0048",  "GeForce 6800 XT"}  ,
		{ "0x0218",  "GeForce 6800 XT"}  ,
		{ "0x00C3",  "GeForce 6800 XT"}  ,
		{ "0x01DF",  "GeForce 7300 GS"}  ,
		{ "0x0393",  "GeForce 7300 GT"}  ,
		{ "0x01D1",  "GeForce 7300 LE"}  ,
		{ "0x01D3",  "GeForce 7300 SE"}  ,
		{ "0x01DD",  "GeForce 7500 LE"}  ,
		{ "0x0392",  "GeForce 7600 GS"}  ,
		{ "0x0392",  "GeForce 7600 GS"}  ,
		{ "0x02E1",  "GeForce 7600 GS"}  ,
		{ "0x0391",  "GeForce 7600 GT"}  ,
		{ "0x0394",  "GeForce 7600 LE"}  ,
		{ "0x00F5",  "GeForce 7800 GS"}  ,
		{ "0x0092",  "GeForce 7800 GT"}  ,
		{ "0x0091",  "GeForce 7800 GTX"}  ,
		{ "0x0291",  "GeForce 7900 GT/GTO"}  ,
		{ "0x0290",  "GeForce 7900 GTX"}  ,
		{ "0x0293",  "GeForce 7900 GX2"}  ,
		{ "0x0294",  "GeForce 7950 GX2"}  ,
		{ "0x0322",  "GeForce FX 5200"}  ,
		{ "0x0321",  "GeForce FX 5200 Ultra"}  ,
		{ "0x0323",  "GeForce FX 5200LE"}  ,
		{ "0x0326",  "GeForce FX 5500"}  ,
		{ "0x0326",  "GeForce FX 5500"}  ,
		{ "0x0312",  "GeForce FX 5600"}  ,
		{ "0x0311",  "GeForce FX 5600 Ultra"}  ,
		{ "0x0314",  "GeForce FX 5600XT"}  ,
		{ "0x0342",  "GeForce FX 5700"}  ,
		{ "0x0341",  "GeForce FX 5700 Ultra"}  ,
		{ "0x0343",  "GeForce FX 5700LE"}  ,
		{ "0x0344",  "GeForce FX 5700VE"}  ,
		{ "0x0302",  "GeForce FX 5800"}  ,
		{ "0x0301",  "GeForce FX 5800 Ultra"}  ,
		{ "0x0331",  "GeForce FX 5900"}  ,
		{ "0x0330",  "GeForce FX 5900 Ultra"}  ,
		{ "0x0333",  "GeForce FX 5950 Ultra"}  ,
		{ "0x0324",  "GeForce FX Go5200 64M"}  ,
		{ "0x031A",  "GeForce FX Go5600"}  ,
		{ "0x0347",  "GeForce FX Go5700"}  ,
		{ "0x0167",  "GeForce Go 6200/6400"}  ,
		{ "0x0168",  "GeForce Go 6200/6400"}  ,
		{ "0x0148",  "GeForce Go 6600"}  ,
		{ "0x00c8",  "GeForce Go 6800"}  ,
		{ "0x00c9",  "GeForce Go 6800 Ultra"}  ,
		{ "0x0098",  "GeForce Go 7800"}  ,
		{ "0x0099",  "GeForce Go 7800 GTX"}  ,
		{ "0x0298",  "GeForce Go 7900 GS"}  ,
		{ "0x0299",  "GeForce Go 7900 GTX"}  ,
		{ "0x0185",  "GeForce MX 4000"}  ,
		{ "0x00FA",  "GeForce PCX 5750"}  ,
		{ "0x00FB",  "GeForce PCX 5900"}  ,
		{ "0x0110",  "GeForce2 MX/MX 400"}  ,
		{ "0x0111",  "GeForce2 MX200"}  ,
		{ "0x0110",  "GeForce2 MX/MX 400"}  ,
		{ "0x0200",  "GeForce3"}  ,
		{ "0x0201",  "GeForce3 Ti200"}  ,
		{ "0x0202",  "GeForce3 Ti500"}  ,
		{ "0x0172",  "GeForce4 MX 420"}  ,
		{ "0x0171",  "GeForce4 MX 440"}  ,
		{ "0x0181",  "GeForce4 MX 440 with AGP8X"}  ,
		{ "0x0173",  "GeForce4 MX 440-SE"}  ,
		{ "0x0170",  "GeForce4 MX 460"}  ,
		{ "0x0253",  "GeForce4 Ti 4200"}  ,
		{ "0x0281",  "GeForce4 Ti 4200 with AGP8X"}  ,
		{ "0x0251",  "GeForce4 Ti 4400"}  ,
		{ "0x0250",  "GeForce4 Ti 4600"}  ,
		{ "0x0280",  "GeForce4 Ti 4800"}  ,
		{ "0x0282",  "GeForce4 Ti 4800SE"}  ,
		{ "0x0203",  "Quadro DCC"}  ,
		{ "0x0309",  "Quadro FX 1000"}  ,
		{ "0x034E",  "Quadro FX 1100"}  ,
		{ "0x00FE",  "Quadro FX 1300"}  ,
		{ "0x00CE",  "Quadro FX 1400"}  ,
		{ "0x0308",  "Quadro FX 2000"}  ,
		{ "0x0338",  "Quadro FX 3000"}  ,
		{ "0x00FD",  "Quadro PCI-E Series"}  ,
		{ "0x00F8",  "Quadro FX 3400/4400"}  ,
		{ "0x00CD",  "Quadro FX 3450/4000 SDI"}  ,
		{ "0x004E",  "Quadro FX 4000"}  ,
		{ "0x00CD",  "Quadro FX 3450/4000 SDI"}  ,
		{ "0x00F8",  "Quadro FX 3400/4400"}  ,
		{ "0x009D",  "Quadro FX 4500"}  ,
		{ "0x029F",  "Quadro FX 4500 X2"}  ,
		{ "0x032B",  "Quadro FX 500/FX 600"}  ,
		{ "0x014E",  "Quadro FX 540"}  ,
		{ "0x014C",  "Quadro FX 540 MXM"}  ,
		{ "0x032B",  "Quadro FX 500/FX 600"}  ,
		{ "0x033F",  "Quadro FX 700"}  ,
		{ "0x034C",  "Quadro FX Go1000"}  ,
		{ "0x00CC",  "Quadro FX Go1400"}  ,
		{ "0x031C",  "Quadro FX Go700"}  ,
		{ "0x018A",  "Quadro NVS with AGP8X"}  ,
		{ "0x032A",  "Quadro NVS 280 PCI"}  ,
		{ "0x00FD",  "Quadro PCI-E Series"}  ,
		{ "0x0165",  "Quadro NVS 285"}  ,
		{ "0x017A",  "Quadro NVS"}  ,
		{ "0x018A",  "Quadro NVS with AGP8X"}  ,
		{ "0x0113",  "Quadro2 MXR/EX"}  ,
		{ "0x017A",  "Quadro NVS"}  ,
		{ "0x018B",  "Quadro4 380 XGL"}  ,
		{ "0x0178",  "Quadro4 550 XGL"}  ,
		{ "0x0188",  "Quadro4 580 XGL"}  ,
		{ "0x025B",  "Quadro4 700 XGL"}  ,
		{ "0x0259",  "Quadro4 750 XGL"}  ,
		{ "0x0258",  "Quadro4 900 XGL"}  ,
		{ "0x0288",  "Quadro4 980 XGL"}  ,
		{ "0x028C",  "Quadro4 Go700"}  ,
		{ "0x0295",  "NVIDIA GeForce 7950 GT"}  ,
		{ "0x03D0",  "NVIDIA GeForce 6100 nForce 430"}  ,
		{ "0x03D1",  "NVIDIA GeForce 6100 nForce 405"}  ,
		{ "0x03D2",  "NVIDIA GeForce 6100 nForce 400"}  ,
		{ "0x0241",  "NVIDIA GeForce 6150 LE"}  ,
		{ "0x0242",  "NVIDIA GeForce 6100"}  ,
		{ "0x0245",  "NVIDIA Quadro NVS 210S / NVIDIA GeForce 6150LE"}  ,
		{ "0x029C",  "NVIDIA Quadro FX 5500"}
	};
	
	int nrModels = sizeof(modelNames)/sizeof(modelNames[0]);
	int i;
	OSString *result = 0;

	for (i=0; i < nrModels; i++) {
		if (venId->isEqualTo(modelNames[i][0])) {
			result = OSString::withCString(modelNames[i][1]);
			break;
		}
	}
	if (result == 0) {
		result = OSString::withCString("Unknown nVidia");
	}
	return result;
}
		

